/* global chrome, FeedParser */
//let ESLint know about global variables that it cannot see being defined

//TODO: show copyright the same way as other stuff,
//sanitised the same way as item bodies
window.onload = () => {
  showPreview();
};

const showPreview = () => {
  //get the URL in a way that the page cannot manipulate
  //prevents pages from using this component to make cross origin requests
  //(it cannot see the result, but it can bypass any cross network
  //restrictions to exploit local network weaknesses)
  chrome.runtime.sendMessage(null,
                             {command: 'get_tab_url'}, {},
                             function (feedURL) {
    if (!feedURL || !feedURL.match(/^(https?|file):/i)) {
      chrome.runtime.sendMessage(
        {command: 'remove_this_preview'}
      );
      return;
    }
    feedURL = feedURL.replace(/#[\w\W]*$/, '');
    const xhr = new XMLHttpRequest();
    xhr.onload = function () {
      let feedParser = new FeedParser();
      let parsed = feedParser.parseFeed(
        xhr.responseXML,
        xhr.responseText,
        feedURL
      );
      if (typeof(parsed) == 'string') {
        chrome.runtime.sendMessage(
          {command: 'remove_this_preview'}
        );
      } else {
        const header = document.createElement('h1');
        header.appendChild(document.createTextNode(parsed.plaintitle));
        document.body.appendChild(header);
        const form = document.createElement('form');
        form.appendChild(document.createElement('input'));
        form.lastChild.type = 'button';
        form.lastChild.value = 'Subscribe';
        form.lastChild.onclick = function () {
          chrome.runtime.sendMessage({
            addFeed: true,
            url: feedURL,
            title: parsed.plaintitle,
            refreshInterval: parsed.ttl
          });
        };
        document.body.appendChild(form);
        const items = document.createElement('ul');
        for (let i = 0; i < parsed.feeditems.length; i++) {
          let li = document.createElement('li');
          items.appendChild(li);
          let subhead = document.createElement('h2');
          subhead.appendChild(document.createElement('a'));
          subhead.lastChild.appendChild(document.createTextNode(
            parsed.feeditems[i].plaintitle
          ));
          if (parsed.feeditems[i].links.alternate.length) {
//TODO: sanitising!
//TODO: in theory, this should only link to the correct language...
//TODO: do we pass extension IDs in the referer header?
            subhead.lastChild.href = parsed.feeditems[i].links.alternate[0].url;
          }
          li.appendChild(subhead);
          let compdate = parsed.feeditems[i].updatedate ||
            parsed.feeditems[i].publishdate;
          if (compdate) {
            compdate = new Date(compdate);
            if (compdate.getTime()) {
              let date = document.createElement('time');
              date.appendChild(document.createTextNode(compdate));
              li.appendChild(date);
            }
          }
          let summary = document.createElement('div');
          //innerHTML is the "wrong" thing to use, since it can potentially
          //contain XML or XHTML too - DOMParser is the right thing to use
          //however, the markup is going to be stripped anyway, so it doesn't
          //matter
          summary.innerHTML = parsed.feeditems[i].htmlsummary ||
            parsed.feeditems[i].htmlbody;
//TODO: decide if sanitising would be better to allow some HTML - it can really
//mess with the layout if wide elements are used
          summary.textContent = summary.textContent;
          li.appendChild(summary);
        }
        document.body.appendChild(items);
        //make sure keyboard works for scrolling
        window.focus();
      }
    };
    xhr.onerror = function () {
      chrome.runtime.sendMessage(
        {command: 'remove_this_preview' }
      );
    };
    xhr.open('GET', feedURL, true);
    xhr.send(null);
  });
};
