/* global chrome, FeedParser */
//let ESLint know about global variables that it cannot see being defined

window.onload = function () {
  function addItems (parentObject, parentElement) {
    for (let i = 0; i < parentObject.items.length; i++) {
      let li = document.createElement('li');
      parentElement.appendChild(li);
      let feedheader = document.createElement('h2');
      li.appendChild(feedheader);
      if (
        parentObject.items[i].feedurl &&
        parentObject.items[i].feedurl.match(/^(https?|file):/i)
      ) {
        let feedlabel = document.createElement('label');
        feedheader.appendChild(feedlabel);
        let feedcheck = document.createElement('input');
        feedcheck.onchange = toggleEnable;
        feedcheck.type = 'checkbox';
        feedlabel.appendChild(feedcheck);
        feedlabel.appendChild(document.createTextNode(
          ' ' + parentObject.items[i].plaintitle
        ));
        let feedlink = document.createElement('a');
        feedlink.href = parentObject.items[i].feedurl;
        feedlink.appendChild(document.createTextNode('Preview'));
        if (parentObject.items[i].language) {
          feedlink.setAttribute('hreflang', parentObject.items[i].language);
        }
        li.appendChild(feedlink);
        checkList.push({
          title: parentObject.items[i].plaintitle,
          url: parentObject.items[i].feedurl,
          checkbox: feedcheck
        });
      } else {
        feedheader.appendChild(document.createTextNode(
          parentObject.items[i].plaintitle
        ));
      }
      if (parentObject.items[i].pageurl) {
        let pagelink = document.createElement('a');
        pagelink.href = parentObject.items[i].pageurl;
        pagelink.appendChild(document.createTextNode('Website'));
//TODO: sanitize!
        li.appendChild(pagelink);
      }
      if (parentObject.items[i].created) {
          let credate = new Date(parentObject.items[i].created);
          if (credate.getTime()) {
            let feeddate = document.createElement('time');
            feeddate.appendChild(document.createTextNode(credate));
            li.appendChild(feeddate);
          }
      }
      if (!parentObject.items[i].supported) {
        li.appendChild(document.createTextNode(
          ' - may not be a supported feed format'
        ));
      }
      if (parentObject.items[i].htmldescription) {
        let summary = document.createElement('div');
        //innerHTML is the "wrong" thing to use, since it can potentially
        //contain XML or XHTML too - DOMParser is the right thing to use
        //however, the markup is going to be stripped anyway, so it doesn't
        //matter
        summary.innerHTML = parentObject.items[i].htmldescription;
        summary.textContent = summary.textContent;
        li.appendChild(summary);
      }
      if (parentObject.items[i].items.length) {
        const subitems = document.createElement('ul');
        addItems(parentObject.items[i], subitems);
        li.appendChild(subitems);
      }
    }
  }
  function toggleEnable() {
    if (subscribeButton) {
      subscribeButton.disabled = !checkList.some(function (checkItem) {
        return checkItem.checkbox.checked;
      });
    }
  }
  const checkList = [];
  let subscribeButton;

//TODO: reverence the variable directly once it is available, and
//remove the sendMessage function below
	let _tabId = location.hash.substring(1);

	//get the URL in a way that the page cannot manipulate
	//prevents pages from using this component to make cross origin requests
	//(it cannot see the result, but it can bypass any cross network
	//restrictions to exploit local network weaknesses)
  chrome.runtime.sendMessage(null,
                             { tabId: _tabId, command: 'get_tab_url' }, {},
                             function (opmlURL) {
    if (!opmlURL || !opmlURL.match(/^(https?|file):/i)) {
      chrome.runtime.sendMessage(
        {tabId: _tabId, command: 'remove_this_preview' }
      );
      return;
    }
    opmlURL = opmlURL.replace(/#[\w\W]*$/, '');
    const xhr = new XMLHttpRequest();
    xhr.onload = function () {
      let feedParser = new FeedParser();
      let parsed = feedParser.parseOPML(xhr.responseXML, opmlURL);
      if (typeof(parsed) == 'string') {
        chrome.runtime.sendMessage(
          {tabId: _tabId, command: 'remove_this_preview' }
        );
      } else {
        const header = document.createElement('h1');
        header.appendChild(document.createTextNode(parsed.plaintitle));
        document.body.appendChild(header);
        const form = document.createElement('form');
        form.appendChild(document.createElement('input'));
        form.lastChild.type = 'button';
        form.lastChild.value = 'Select all';
        form.lastChild.onclick = function () {
          for (let i = 0; i < checkList.length; i++ ) {
            checkList[i].checkbox.checked = true;
          }
          toggleEnable();
        };
        form.appendChild(document.createTextNode(' '));
        form.appendChild(document.createElement('input'));
        form.lastChild.type = 'button';
        form.lastChild.value = 'De-select all';
        form.lastChild.onclick = function () {
          for (let i = 0; i < checkList.length; i++ ) {
            checkList[i].checkbox.checked = false;
          }
          toggleEnable();
        };
        form.appendChild(document.createTextNode(' '));
        subscribeButton = document.createElement('input');
        form.appendChild(subscribeButton);
        form.lastChild.type = 'button';
        form.lastChild.value = 'Subscribe to selected feeds';
        form.lastChild.onclick = function () {
          const allSelected = [];
          for (let i = 0; i < checkList.length; i++ ) {
            if (checkList[i].checkbox.checked) {
              allSelected.push(checkList[i]);
//TODO: hook in the function to add the selected feeds
            }
          }
        };
        document.body.appendChild(form);
        const items = document.createElement('ul');
        items.className = 'nested';
        addItems(parsed, items);
        toggleEnable();
        document.body.appendChild(items);
        //make sure keyboard works for scrolling
        window.focus();
      }
    };
    xhr.onerror = function () {
      chrome.runtime.sendMessage(
        {tabId: _tabId, command: 'remove_this_preview' }
      );
    };
    xhr.open('GET', opmlURL, true);
    xhr.send(null);
  });
};